#!/usr/bin/python
# File: gmpm/__init__.py
# This file is part of yapyb
#
# Copyright (C) 2008 Juan Fiol
# This is free software.  You may redistribute copies of it under the terms of
# the GNU General Public License version 2.
# There is NO WARRANTY, to the extent permitted by law.
#
# Written by Juan Fiol <juanfiol@gmail.com>


__all__ = ["query_ads"]
